@@ -21,11 +21,12 @@ from label_studio.utils.models import ProjectObj, MLBackend
 from label_studio.utils.exceptions import ValidationError
 from label_studio.utils.io import find_file, delete_dir_content, json_load
 from label_studio.utils.validation import is_url
-from label_studio.utils.functions import get_external_hostname
 from label_studio.tasks import Tasks
 from label_studio.storage import create_storage, get_available_storage_names
+from label_studio.utils.misc import Settings
 
 logger = logging.getLogger(__name__)
+settings = Settings()
 
 
 class ProjectNotFound(KeyError):
@@ -78,8 +79,9 @@ class Project(object):
         self.target_storage = None
         self.create_storages()
 
-        self.label_config_line, self.label_config_full, self.parsed_label_config, self.input_data_tags = None, None, None, None  # noqa
-        self.derived_input_schema, self.derived_output_schema = None, None
+        self.label_config_line, self.label_config_full, self.parsed_label_config = None, None, None  # noqa
+        self.derived_input_schema, self.derived_output_schema, self.derived_all_input_schema = None, None, None
+        self.config_input_tags = None
 
         self.load_label_config()
         self.load_project_and_ml_backends()
@@ -108,7 +110,15 @@ class Project(object):
 
     @property
     def export_dir(self):
-        return os.path.join(self.path, 'export')
+        export_dir = os.path.join(self.path, 'export')
+        os.makedirs(export_dir, exist_ok=True)
+        return export_dir
+
+    @property
+    def upload_dir(self):
+        upload_dir = os.path.join(self.path, 'upload')
+        os.makedirs(upload_dir, exist_ok=True)
+        return upload_dir
 
     def get_storage(self, storage_for):
         if storage_for == 'source':
@@ -197,68 +207,29 @@ class Project(object):
     def data_types_json(self):
         return self.project_obj.data_types_json
 
-    def load_label_config(self):
-        self.label_config_full = config_comments_free(open(self.label_config_path, encoding='utf8').read())
-        self.label_config_line = config_line_stripped(self.label_config_full)
-        self.parsed_label_config = parse_config(self.label_config_line)
-        self.input_data_tags = self.get_input_data_tags(self.label_config_line)
+    @property
+    def no_control_tags(self):
+        return not self.parsed_label_config
 
-    def update_derived_input_schema(self):
-        self.derived_input_schema = set()
-        for task_id, task in self.source_storage.items():
-            data_keys = set(task['data'].keys())
-            if not self.derived_input_schema:
-                self.derived_input_schema = data_keys
-            else:
-                self.derived_input_schema &= data_keys
-        logger.debug('Derived input schema: ' + str(self.derived_input_schema))
+    @property
+    def label_config_is_empty(self):
+        return self.no_control_tags and not self.data_types
 
-    def update_derived_output_schema(self):
-        self.derived_output_schema = {
-            'from_name_to_name_type': set(),
-            'labels': defaultdict(set)
+    @property
+    def one_object_in_label_config(self):
+        return len(self.data_types) <= 1
+
+    @property
+    def empty_derived_input_schema(self):
+        return not self.derived_input_schema
+
+    @property
+    def input_data_scheme(self):
+        return {
+            (x.attrib.get('name', ''), x.attrib.get('value', ''))
+            for x in self.config_input_tags
         }
 
-        # for all already completed tasks we update derived output schema for further label config validation
-        for task_id, c in self.target_storage.items():
-            for completion in c['completions']:
-                self._update_derived_output_schema(completion)
-        logger.debug('Derived output schema: ' + str(self.derived_output_schema))
-
-    def add_ml_backend(self, params, raise_on_error=True):
-        ml_backend = MLBackend.from_params(params)
-        if not ml_backend.connected and raise_on_error:
-            raise ValueError('ML backend with URL: "' + str(params['url']) + '" is not connected.')
-        self.ml_backends.append(ml_backend)
-
-    def remove_ml_backend(self, name):
-        # remove from memory
-        remove_idx = next((i for i, b in enumerate(self.ml_backends) if b.model_name == name), None)
-        if remove_idx is None:
-            raise KeyError('Can\'t remove ML backend with name "' + name + '": not found.')
-        self.ml_backends.pop(remove_idx)
-
-        # remove from config
-        config_params = self.config.get('ml_backends', [])
-        remove_idx = next((i for i, b in enumerate(config_params) if b['name'] == name), None)
-        if remove_idx is not None:
-            config_params.pop(remove_idx)
-        self.config['ml_backends'] = config_params
-        self._save_config()
-
-    def load_project_and_ml_backends(self):
-        # configure project
-        self.project_obj = ProjectObj(label_config=self.label_config_line, label_config_full=self.label_config_full)
-
-        # configure multiple machine learning backends
-        self.ml_backends = []
-        ml_backends_params = self.config.get('ml_backends', [])
-        for ml_backend_params in ml_backends_params:
-            self.add_ml_backend(ml_backend_params, raise_on_error=False)
-
-    def load_converter(self):
-        self.converter = Converter(self.parsed_label_config)
-
     @property
     def id(self):
         return self.project_obj.id
@@ -271,6 +242,10 @@ class Project(object):
     def data_types(self):
         return self.project_obj.data_types
 
+    @property
+    def data_keys(self):
+        return list([key for key in self.derived_input_schema if key != Settings.UPLOAD_DATA_UNDEFINED_NAME])
+
     @property
     def label_config(self):
         return self.project_obj.label_config
@@ -287,32 +262,14 @@ class Project(object):
     def task_data_password(self):
         return self.project_obj.task_data_password
 
-    def extract_data_types(self, config):
-        return self.project_obj.extract_data_types(config)
+    # ============= BEGIN LABEL CONFIG MANAGEMENT SECTION ================
 
-    def validate_label_config(self, config_string):
-        logger.debug('Validate label config')
-        self.project_obj.validate_label_config(config_string)
-
-        logger.debug('Get parsed config')
-        parsed_config = parse_config(config_string)
-
-        logger.debug('Validate label config on derived input schema')
-        self.validate_label_config_on_derived_input_schema(parsed_config)
-
-        logger.debug('Validate label config on derived output schema')
-        self.validate_label_config_on_derived_output_schema(parsed_config)
-
-    def _save_config(self):
-        with io.open(self.config_path, mode='w') as f:
-            json.dump(self.config, f, indent=2)
-
-    def update_params(self, params):
-        if 'ml_backend' in params:
-            ml_backend_params = self._create_ml_backend_params(params['ml_backend'], self.name)
-            self.add_ml_backend(ml_backend_params)
-            self.config['ml_backends'].append(ml_backend_params)
-            self._save_config()
+    def load_label_config(self):
+        with open(self.label_config_path, encoding='utf8') as f:
+            self.label_config_full = config_comments_free(f.read())
+        self.label_config_line = config_line_stripped(self.label_config_full)
+        self.parsed_label_config = parse_config(self.label_config_line)
+        self.config_input_tags = self.get_config_input_tags(self.label_config_line)
 
     def update_label_config(self, new_label_config):
         # save xml label config to file
@@ -332,6 +289,48 @@ class Project(object):
             json.dump(self.config, f)
         logger.info('Label config saved to: {path}'.format(path=self.label_config_path))
 
+    def reset_label_config(self):
+        logger.debug('Reset label config')
+        return self.update_label_config('<View></View>')
+
+    def validate_label_config(self, config_string):
+        logger.debug('Validate label config')
+        self.project_obj.validate_label_config(config_string)
+
+        logger.debug('Get parsed config')
+        parsed_config = parse_config(config_string)
+
+        logger.debug('Validate label config on derived input schema')
+        self.validate_label_config_on_derived_input_schema(parsed_config)
+
+        logger.debug('Validate label config on derived output schema')
+        self.validate_label_config_on_derived_output_schema(parsed_config)
+
+    def update_derived_input_schema(self):
+        self.derived_input_schema = set()
+        for task_id, task in self.source_storage.items():
+            data_keys = set(task['data'].keys())
+            if not self.derived_input_schema:
+                self.derived_input_schema = data_keys
+                self.derived_all_input_schema = data_keys
+            else:
+                self.derived_input_schema &= data_keys
+                self.derived_all_input_schema = self.derived_all_input_schema | data_keys   # ! don't try a |= b
+        logger.debug('Derived input schema: ' + str(self.derived_input_schema))
+        logger.debug('Derived all input schema: ' + str(self.derived_all_input_schema))
+
+    def update_derived_output_schema(self):
+        self.derived_output_schema = {
+            'from_name_to_name_type': set(),
+            'labels': defaultdict(set)
+        }
+
+        # for all already completed tasks we update derived output schema for further label config validation
+        for task_id, c in self.target_storage.items():
+            for completion in c['completions']:
+                self._update_derived_output_schema(completion)
+        logger.debug('Derived output schema: ' + str(self.derived_output_schema))
+
     def _update_derived_output_schema(self, completion):
         """
         Given completion, output schema is updated. Output schema consists of unique tuples (from_name, to_name, type)
@@ -351,7 +350,10 @@ class Project(object):
                 result['from_name'], result['to_name'], result_type
             ))
             for label in result['value'].get(result_type, []):
-                self.derived_output_schema['labels'][result['from_name']].add(label)
+                if not isinstance(label, list):
+                    label = [label]
+                for l in label:
+                    self.derived_output_schema['labels'][result['from_name']].add(l)
 
     def validate_label_config_on_derived_input_schema(self, config_string_or_parsed_config):
         """
@@ -361,7 +363,9 @@ class Project(object):
         """
 
         # check if schema exists, i.e. at least one task has been uploaded
-        if not self.derived_input_schema:
+        if not self.derived_input_schema or \
+                (len(self.derived_input_schema) == 1 and
+                 next(iter(self.derived_input_schema)) == settings.UPLOAD_DATA_UNDEFINED_NAME):
             return
 
         config = config_string_or_parsed_config
@@ -377,9 +381,8 @@ class Project(object):
         for item in input_values:
             if item not in self.derived_input_schema:
                 raise ValidationError(
-                    'You have already imported tasks and they are incompatible with a new config. '
-                    'You\'ve specified value=${item}, but imported tasks contain only keys: {input_schema_values}'
-                        .format(item=item, input_schema_values=list(self.derived_input_schema)))
+                    'Please replace "{0}" in value=${0} with one of the following columns:\n- {1}'.format(
+                        item, '\n- '.join(sorted(self.derived_input_schema))))
 
     def validate_label_config_on_derived_output_schema(self, config_string_or_parsed_config):
         """
@@ -422,10 +425,60 @@ class Project(object):
                     .format(from_name=from_name, extra_labels=extra_labels)
                 )
 
+    # ============= END LABEL CONFIG MANAGEMENT SECTION ================
+
+    def add_ml_backend(self, params, raise_on_error=True):
+        ml_backend = MLBackend.from_params(params)
+        if not ml_backend.connected and raise_on_error:
+            raise ValueError('ML backend with URL: "' + str(params['url']) + '" is not connected.')
+        self.ml_backends.append(ml_backend)
+
+    def remove_ml_backend(self, name):
+        # remove from memory
+        remove_idx = next((i for i, b in enumerate(self.ml_backends) if b.model_name == name), None)
+        if remove_idx is None:
+            raise KeyError('Can\'t remove ML backend with name "' + name + '": not found.')
+        self.ml_backends.pop(remove_idx)
+
+        # remove from config
+        config_params = self.config.get('ml_backends', [])
+        remove_idx = next((i for i, b in enumerate(config_params) if b['name'] == name), None)
+        if remove_idx is not None:
+            config_params.pop(remove_idx)
+        self.config['ml_backends'] = config_params
+        self._save_config()
+
+    def load_project_and_ml_backends(self):
+        # configure project
+        self.project_obj = ProjectObj(label_config=self.label_config_line, label_config_full=self.label_config_full)
+
+        # configure multiple machine learning backends
+        self.ml_backends = []
+        ml_backends_params = self.config.get('ml_backends', [])
+        for ml_backend_params in ml_backends_params:
+            self.add_ml_backend(ml_backend_params, raise_on_error=False)
+
+    def load_converter(self):
+        self.converter = Converter(self.parsed_label_config)
+
+    def extract_data_types(self, config):
+        return self.project_obj.extract_data_types(config)
+
+    def _save_config(self):
+        with io.open(self.config_path, mode='w') as f:
+            json.dump(self.config, f, indent=2)
+
+    def update_params(self, params):
+        if 'ml_backend' in params:
+            ml_backend_params = self._create_ml_backend_params(params['ml_backend'], self.name)
+            self.add_ml_backend(ml_backend_params)
+            self.config['ml_backends'].append(ml_backend_params)
+            self._save_config()
+
     def no_tasks(self):
         return self.source_storage.empty()
 
-    def delete_tasks(self):
+    def delete_all_tasks(self):
         """
         Deletes all tasks & completions from filesystem, then reloads clean project
         :return:
@@ -440,19 +493,27 @@ class Project(object):
             for m in self.ml_backends:
                 m.clear(self)
 
-    def next_task(self, completed_tasks_ids):
-        completed_tasks_ids = set(completed_tasks_ids)
-        sampling = self.config.get('sampling', 'sequential')
+    def next_task(self, completed_tasks_ids, task_ids=None, sampling=None):
+        """ Generate next task for labeling
 
-        # Tasks are ordered ascending by their "id" fields. This is default mode.
-        task_iter = filter(lambda i: i not in completed_tasks_ids, sorted(self.source_storage.ids()))
+            :param completed_tasks_ids: ids of tasks that have completions
+            :param task_ids: array of dicts with tasks (dict.items())
+            :param sampling: string with sampling names: sequential, random-uniform,
+                             prediction-score-max, prediction-score-min
+        """
+        completed_tasks_ids = set(completed_tasks_ids)
+        task_ids = sorted(self.source_storage.ids()) if task_ids is None else task_ids
+        sampling = self.config.get('sampling', 'sequential') if sampling is None else sampling
+
+        # Tasks are ordered ascending by their "id" fields. This is default mode
+        task_iter = filter(lambda i: i not in completed_tasks_ids, task_ids)
         if sampling == 'sequential':
             task_id = next(task_iter, None)
             if task_id is not None:
                 return self.source_storage.get(task_id)
 
         # Tasks are sampled with equal probabilities
-        elif sampling == 'uniform':
+        elif sampling == 'uniform' or sampling == 'random-uniform':
             actual_tasks_ids = list(task_iter)
             if not actual_tasks_ids:
                 return None
@@ -462,76 +523,45 @@ class Project(object):
         # Task with minimum / maximum average prediction score is taken
         elif sampling.startswith('prediction-score'):
             id_score_map = {}
-            for task_id, task in self.source_storage.items():
-                if task_id in completed_tasks_ids:
+
+            for task_id in task_ids:
+                # get task from storage and check it has completions
+                task = self.source_storage.get(task_id)
+                if task_id in completed_tasks_ids or task is None:
                     continue
+                # average all predictions and save score
                 if 'predictions' in task and len(task['predictions']) > 0:
                     score = sum((p['score'] for p in task['predictions']), 0) / len(task['predictions'])
                     id_score_map[task_id] = score
+
             if not id_score_map:
                 return None
+
             if sampling.endswith('-min'):
                 best_idx = min(id_score_map, key=id_score_map.get)
             elif sampling.endswith('-max'):
                 best_idx = max(id_score_map, key=id_score_map.get)
             else:
                 raise NotImplementedError('Unknown sampling method ' + sampling)
+
             return self.source_storage.get(best_idx)
         else:
             raise NotImplementedError('Unknown sampling method ' + sampling)
 
-    def remove_task(self, task_id):
-        self.source_storage.remove(task_id)
-        self.delete_task_completions(task_id)
-
-        self.update_derived_input_schema()
-        self.update_derived_output_schema()
-
-    def get_completions_ids(self):
-        """ List completion ids from output_dir directory
+    def get_completions_ids(self, task_ids=None):
+        """ List completion files with ids from output_dir directory
 
         :return: filenames without extensions and directories
         """
-        task_ids = set(self.source_storage.ids())
+        if task_ids is None:
+            task_ids = set(self.source_storage.ids())
         completion_ids = set(self.target_storage.ids())
         completions = completion_ids.intersection(task_ids)
-        #completions = list(self.target_storage.ids())
+
         logger.debug('{num} completions found in {output_dir}'.format(
             num=len(completions), output_dir=self.config["output_dir"]))
         return sorted(completions)
 
-    def get_completed_at(self):
-        """ Get completed time for tasks
-
-        :return: list of string with formatted datetime
-        """
-        times = {}
-        for _, data in self.target_storage.items():
-            id = data['id']
-            try:
-                times[id] = max(data['completions'], key=itemgetter('created_at'))['created_at']
-            except Exception as exc:
-                times[id] = 'undefined'
-        return times
-
-    def get_cancelled_status(self):
-        """ Get was_cancelled (skipped) status for tasks: returns cancelled completion number for task
-
-        :return: list of int
-        """
-        items = {}
-        for _, data in self.target_storage.items():
-            id = data['id']
-            try:
-                # note: skipped will be deprecated
-                flag = sum([completion.get('skipped', False) or completion.get('was_cancelled', False)
-                            for completion in data['completions']])
-            except Exception as exc:
-                items[id] = -1
-            else:
-                items[id] = flag
-        return items
-
     def get_task_with_completions(self, task_id):
         """ Get task with completions
 
@@ -543,8 +573,8 @@ class Project(object):
 
         if data:
             logger.debug('Get predictions ' + str(task_id) + ' from source storage')
-            # tasks can hold the newest version of predictions, so task it from tasks
-            data['predictions'] = self.source_storage.get(task_id).get('predictions', [])
+            # tasks can hold the newest version of predictions, so get it from tasks
+            data['predictions'] = self.source_storage.get(task_id, inplace=True, validate=False).get('predictions', [])
         return data
 
     def save_completion(self, task_id, completion):
@@ -559,6 +589,7 @@ class Project(object):
         # init task if completions with task not exists
         if not task:
             task = deepcopy(self.source_storage.get(task_id))
+            task = self.resolve_undefined_task_data(task)
             task['completions'] = []
         else:
             task = deepcopy(task)
@@ -570,11 +601,16 @@ class Project(object):
         if 'id' in completion:
             for i, item in enumerate(task['completions']):
                 if item['id'] == completion['id']:
+                    # we don't want to explicitly make "was_cancelled" key if it's not presented in storage
+                    if 'was_cancelled' in task['completions'][i] and not completion.get('was_cancelled', False):
+                        task['completions'][i]['was_cancelled'] = False
                     task['completions'][i].update(completion)
                     updated = True
-        # write new completion
+        # write a new completion
         if not updated:
-            completion['id'] = task['id'] * 1000 + len(task['completions']) + 1
+            # start completion id from task_id * 1000
+            completions = task.get('completions', None) or [{'id': task_id * 1000}]
+            completion['id'] = max([c['id'] for c in completions]) + 1
             task['completions'].append(completion)
 
         try:
@@ -591,8 +627,30 @@ class Project(object):
         logger.debug('Completion for task ' + str(task_id) + ' saved with id =' + str(completion['id']))
         return completion['id']
 
+    def delete_task(self, task_id):
+        """ Delete one task
+        """
+        self.source_storage.remove(task_id)
+        self.delete_task_completions(task_id)
+
+        self.update_derived_input_schema()
+        self.update_derived_output_schema()
+
+    def delete_tasks(self, task_ids):
+        """ Delete by list of task ids
+        """
+        self.source_storage.remove_all(task_ids)
+        self.target_storage.remove_all(task_ids)
+
+        self.update_derived_input_schema()
+        self.update_derived_output_schema()
+
+        if self.empty_derived_input_schema and self.no_control_tags:
+            # after removing all input tasks, reset config consisted of object tags only for future uploads
+            self.reset_label_config()
+
     def delete_task_completion(self, task_id, completion_id):
-        """ Delete one task completion by id
+        """ Delete one completion by id
         """
         # try to get completions with task first
         task = self.get_task_with_completions(task_id)
@@ -607,19 +665,30 @@ class Project(object):
             if item['id'] == completion_id:
                 del task['completions'][i]
 
-        self.update_derived_output_schema()
+        # remove task from completions storage if there are no completions at all
+        if len(task['completions']) == 0:
+            self.target_storage.remove(task_id)
+        # write task to completions storage
+        else:
+            self.target_storage.set(task_id, task)
 
-        # write task + completions to file
-        self.target_storage.set(task_id, task)
         logger.debug('Completion ' + str(completion_id) + ' removed:\n')
+        self.update_derived_output_schema()
         return True
 
     def delete_task_completions(self, task_id):
-        """ Delete all task completions
+        """ Delete all completions for one task
         """
         self.target_storage.remove(task_id)
         self.update_derived_output_schema()
 
+    def delete_tasks_completions(self, task_ids):
+        """ Delete all completions for list of task ids
+        """
+        for task_id in task_ids:
+            self.target_storage.remove(task_id)
+        self.update_derived_output_schema()
+
     def delete_all_completions(self):
         """ Delete all completions from project
         """
@@ -661,7 +730,7 @@ class Project(object):
         return os.path.join(args.root_dir, project_name)
 
     @classmethod
-    def get_input_data_tags(cls, label_config):
+    def get_config_input_tags(cls, label_config):
         tag_iter = ElementTree.fromstring(label_config).iter()
         return [
             tag for tag in tag_iter
@@ -678,19 +747,19 @@ class Project(object):
             return task_loader.from_json_file(input_path)
         if args.input_format == 'json-dir':
             return task_loader.from_dir_with_json_files(input_path)
-        input_data_tags = cls.get_input_data_tags(label_config)
+        config_input_tags = cls.get_config_input_tags(label_config)
 
-        if len(input_data_tags) > 1:
-            val = ",".join(tag.attrib.get("name") for tag in input_data_tags)
+        if len(config_input_tags) > 1:
+            val = ",".join(tag.attrib.get("name") for tag in config_input_tags)
             print('Warning! Multiple input data tags found: ' +
                   val + '. Only first one is used.')
-        elif len(input_data_tags) == 0:
+        elif len(config_input_tags) == 0:
             raise ValueError(
                 'You\'ve specified input format "{fmt}" which requires label config being explicitly defined. '
                 'Please specify --label-config=path/to/config.xml or use --format=json or format=json_dir'.format(
                     fmt=args.input_format)
             )
-        input_data_tag = input_data_tags[0]
+        input_data_tag = config_input_tags[0]
         data_key = input_data_tag.attrib.get('value').lstrip('$')
 
         if args.input_format == 'text':
@@ -725,23 +794,26 @@ class Project(object):
         :return:
         """
         dir = cls.get_project_dir(project_name, args)
-        if args.force:
+        if hasattr(args, 'force') and args.force:
             delete_dir_content(dir)
         os.makedirs(dir, exist_ok=True)
 
-        config = json_load(args.config_path) if args.config_path else json_load(find_file('default_config.json'))
+        if hasattr(args, 'config_path') and args.config_path:
+            config = json_load(args.config_path)
+        else:
+            config = json_load(find_file('default_config.json'))
 
         def already_exists_error(what, path):
             raise RuntimeError('{path} {what} already exists. Use "--force" option to recreate it.'.format(
                 path=path, what=what
             ))
 
-        input_path = args.input_path or config.get('input_path')
+        input_path = hasattr(args, 'input_path') and args.input_path or config.get('input_path')
 
         # save label config
         config_xml = 'config.xml'
         config_xml_path = os.path.join(dir, config_xml)
-        label_config_file = args.label_config or config.get('label_config')
+        label_config_file = hasattr(args, 'label_config') and args.label_config or config.get('label_config')
         if label_config_file:
             copy2(label_config_file, config_xml_path)
             print(label_config_file + ' label config copied to ' + config_xml_path)
@@ -750,7 +822,7 @@ class Project(object):
                 already_exists_error('label config', config_xml_path)
             if not input_path:
                 # create default config with polygons only if input data is not set
-                default_label_config = find_file('examples/image_polygons/config.xml')
+                default_label_config = find_file('examples/starter.xml')
                 copy2(default_label_config, config_xml_path)
                 print(default_label_config + ' label config copied to ' + config_xml_path)
             else:
@@ -760,7 +832,7 @@ class Project(object):
 
         config['label_config'] = config_xml
 
-        if args.source:
+        if hasattr(args, 'source') and args.source:
             config['source'] = {
                 'type': args.source,
                 'path': args.source_path,
@@ -785,7 +857,7 @@ class Project(object):
             logger.debug('{tasks_json_path} input file with {n} tasks has been created from {input_path}'.format(
                 tasks_json_path=tasks_json_path, n=len(tasks), input_path=input_path))
 
-        if args.target:
+        if hasattr(args, 'target') and args.target:
             config['target'] = {
                 'type': args.target,
                 'path': args.target_path,
@@ -810,23 +882,24 @@ class Project(object):
 
         if 'ml_backends' not in config or not isinstance(config['ml_backends'], list):
             config['ml_backends'] = []
-        if args.ml_backends:
+        if hasattr(args, 'ml_backends') and args.ml_backends:
             for url in args.ml_backends:
                 config['ml_backends'].append(cls._create_ml_backend_params(url, project_name))
 
-        if args.sampling:
+        if hasattr(args, 'sampling') and args.sampling:
             config['sampling'] = args.sampling
-        if args.port:
+        if hasattr(args, 'port') and args.port:
             config['port'] = args.port
-        if args.host:
+        if hasattr(args, 'host') and args.host:
             config['host'] = args.host
-        if args.allow_serving_local_files:
+        if hasattr(args, 'allow_serving_local_files') and args.allow_serving_local_files:
             config['allow_serving_local_files'] = True
-        if args.key_file and args.cert_file:
+        if hasattr(args, 'key_file') and args.key_file and args.cert_file:
             config['protocol'] = 'https://'
             config['cert'] = args.cert_file
             config['key'] = args.key_file
-        if (hasattr(args, 'web_gui_project_desc') and args.web_gui_project_desc) or args.project_desc:
+        if (hasattr(args, 'web_gui_project_desc') and args.web_gui_project_desc) or \
+            (hasattr(args, 'project_desc') and args.project_desc):
             config['description'] = args.web_gui_project_desc or args.project_desc
 
         # create config.json
@@ -941,16 +1014,29 @@ class Project(object):
     def supported_formats(self):
         return self.project_obj.supported_formats
 
+    def resolve_undefined_task_data(self, task):
+        """ Resolve task data with $undefined$ inplace
+        """
+        # resolve special reserved $undefined$ key
+        if self.data_types:
+            new_data = dict(task['data'])
+            new_key = next(iter(self.data_types))
+            value = new_data.pop(settings.UPLOAD_DATA_UNDEFINED_NAME, None)
+            if value is not None:
+                new_data[new_key] = value
+            task['data'] = new_data
+        return task
+
     def serialize(self):
         """ Serialize project to json dict
         """
         ban_list = ('json', 'dir-jsons')
         available_storages = list(filter(lambda i: i[0] not in ban_list, get_available_storage_names().items()))
-
+        task_ids = self.source_storage.ids()
         output = {
             'project_name': self.name,
-            'task_count': len(self.source_storage.ids()),
-            'completion_count': len(self.get_completions_ids()),
+            'task_count': len(task_ids),
+            'completion_count': len(self.get_completions_ids(task_ids)),
             'config': self.config,
             'instruction': self.config['instruction'],
             'can_manage_tasks': self.can_manage_tasks,
@@ -962,6 +1048,7 @@ class Project(object):
             'source_syncing': self.source_storage.is_syncing,
             'target_syncing': self.target_storage.is_syncing,
             'data_types': self.data_types,
-            'label_config_line': self.label_config_line
+            'label_config_line': self.label_config_line,
+            'config_has_control_tags': len(self.parsed_label_config) > 0
         }
         return output
