@@ -28,6 +28,8 @@ def action_config(test_client, case_config):
         'label_config': case_config['label_config']
     }
     response = test_client.post('/api/project/config', data=data, headers=headers)
+    if response.status_code != 201:
+        print('\n\n\n -- response.data -->', response.data, '\n\n\n')
     assert response.status_code == 201
 
 
@@ -89,10 +91,10 @@ def action_next_task(test_client, case_config):
         action
         get all tasks
     """
-    #TODO get tasks
-    response = test_client.get('/api/project/next')
+    # TODO get tasks
+    response = test_client.get('/api/project/actions?id=next_task')
     data = json.loads(response.data.decode('utf-8'))
-    assert isinstance(data, dict) == True
+    assert isinstance(data, dict)
 
 
 def action_get_task(test_client, case_config):
